/*
 * Execve is a DOS program that runs other DOS programs after
 * setting variables for the child program's environment.
 *
 * Syntax:
 *     execve [-Estring=value ...] [-Ffilename] prog.exe [parameters]
 *
 * The -E switches let you specify environment variables and their
 * values on the command line.  The -F switch specifies a file from
 * which to read environment variable settings.
 *
 * There must be no space following the -E and -F options.
 * The file designated by the optional -F switch must contain lines
 * of the form
 *
 *     string=value
 * 
 * just as given in the -E parameters.
 *
 * If no filename is given with the -F switch, the default file
 * "execve.cfg" in the current directory is used.
 *
 * If the same variable is set from a file given in a -F argument and
 * from a -E argument, the -E setting takes precedence.  If the same
 * variable is set in two -E arguments, the rightmost argument takes
 * precedence.
 *
 * The program name may be any fully-qualified pathname to an executable,
 * or may be just the root name of an executable residing in a
 * directory that the PATH variable points to.  Any changes made to
 * the PATH setting by -E or -F arguments will be in effect.
 * 
 * After the name of the executable go any arguments for the executable.
 * Execve will not interpret any arguments beyond the initial -E and
 * -F switches.
 *
 *
 * Premia Corporation
 * February 1994
 */


#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <process.h>

#define	MAXESTRS	(128)
#define	MAXCFGFILES	(128)
#define	ESTRLENMAX	(128)

void setEnvFromFile(char *fnam);
void setEnvFromString(char *s);

int
main(int argc, char **argv)
{
	char **argvbase = argv;
	
	if (*++argv)
	{
		while (*argv && **argv == '-')
		{
			if (toupper((*argv)[1]) == 'F')
				setEnvFromFile(*argv + 2);
			++argv;	
		}
	}
	
	argv = argvbase;	
		
	if (*++argv)
	{
		while (*argv && **argv == '-')
		{
			if (toupper((*argv)[1]) == 'E')
				setEnvFromString(*argv + 2);
			++argv;	
		}
	}
	
	if (execvp(argv[0], argv))
	{
		perror("Execve");
		return(-1);
	}
	return(0);
}	

/*
 * _putenv will leave envrionment pointing to whatever space
 * is given here.  That's why the strings have to be copied
 * into static space.  The child process inherits that space,
 * so its environment pointers are valid.
 * If we're all out of static space, do nothing.
 */
				 
void				 
setEnvFromString(char *s)
{
	static idx;
	static char estrs[MAXESTRS][ESTRLENMAX];

	if (s && *s)
		if (idx < MAXESTRS)
		{
			strncpy(estrs[idx], s, ESTRLENMAX - 1);
			estrs[idx][ESTRLENMAX - 1] = 0;	
			if (_putenv(estrs[idx]))
				fprintf(stderr, "Error setting %s\n", s);
			++idx;	
		}
}

/*
 * Lines in the file must be of the form
 *  var=value
 * because they are passed to putenv without processing.
 */

void
setEnvFromFile(char *fnam)
{
	FILE *fp;
	static char iobu[ESTRLENMAX];
	char *iop;

	if (!fnam || !*fnam)
		fnam = "execve.cfg";
	if (fp = fopen(fnam, "r"))
	{
		while (fgets(iobu, ESTRLENMAX, fp))
		{
			if (iobu[0])
			{
				if (iop = strchr(iobu, '\r'))
					*iop = 0;
				else if (iop = strchr(iobu, '\n'))
					*iop = 0;
				if (*iobu)
					setEnvFromString(iobu);
			}
		}
		fclose(fp);	
	}	
}
		
		